#define MICROPY_HW_BOARD_NAME "Arduino Nano RP2040 Connect"
#define MICROPY_HW_MCU_NAME "rp2040"

// #define MICROPY_HW_LED_STATUS (&pin_GPIO6)

#define DEFAULT_I2C_BUS_SCL (&pin_GPIO13)
#define DEFAULT_I2C_BUS_SDA (&pin_GPIO12)

#define DEFAULT_SPI_BUS_SCK (&pin_GPIO6)
#define DEFAULT_SPI_BUS_MOSI (&pin_GPIO7)
#define DEFAULT_SPI_BUS_MISO (&pin_GPIO4)

#define DEFAULT_UART_BUS_RX (&pin_GPIO1)
#define DEFAULT_UART_BUS_TX (&pin_GPIO0)
