
#ifndef MICROPY_INCLUDED_SHARED_BINDINGS_COUNTIO___INIT___H
#define MICROPY_INCLUDED_SHARED_BINDINGS_COUNTIO___INIT___H

#include "py/obj.h"

// Nothing now.

#endif  // MICROPY_INCLUDED_SHARED_BINDINGS_COUNTIO___INIT___H
