s = {1, 2, 3, 4}
print(s.add(5))
print(sorted(s))

s = set()
s.add(0)
s.add(False)
print(s)

s = set()
s.add(False)
s.add(0)
print(s)

s = set()
s.add(1)
s.add(True)
print(s)

s = set()
s.add(True)
s.add(1)
print(s)
